Unit mouse;
{----------------------------------------------}
{ Taken from Turbo Technix .. for use on 4.n   }
{ For use with microsoft/logitech compatibles  }
{              NOTE!!!!!!!                     }
{ Compiler Options MUST be set to Far Calls!   }
{ Otherwise the task set by mInstTask will bomb}
{----------------------------------------------}
Interface
Uses DOS;
Const
  MDD = $33;    {Interrupt for mouse driver}

Type
  resetRec = record      {initialized by mouse function 0}
    exists : boolean;    {True if mouse present}
    nButtons : integer;  { # of buttons on mouse (2 or 3) }
  end;
  locRec = record      {initialized by mouse functions 3, 5, and  6 }
    buttonStatus,      {bits 0-2 for corresponding button pressed   }
    opCount,           { # times button clicked ... not set by fcn 3}
    column,
    row      : integer;
  end;
  moveRec = record     {initialized by mouse function 11 }
    hCount,
    vCount : integer   {net horizontal/vertical movements}
  end;

Var   Reg : Registers;    { a la DOS unit}
                          {microsoft mouse driver function calls}
Procedure mReset(var mouse : resetRec);                 {func  0}
Procedure mShow;                                        {func  1}
Procedure mHide;                                        {func  2}
Procedure mPos(var mouse : locRec);                     {func  3}
Procedure mTxtPos(var mouse : locRec);                  {func  3}
Procedure mMoveto(col, row : integer);                  {func  4}
Procedure mTxtMoveto(col, row : integer);               {func  4}
Procedure mPressed(button : integer;
                   var mouse : locRec);                 {func  5}
Procedure mPressedT(button : integer;
                   var mouse : locRec);                 {func  5}
Procedure mReleased(button : integer;
                   var mouse : locRec);                 {func  6}
Procedure mReleasedT(button : integer;
                   var mouse : locRec);                 {func  6}
Procedure mColRange(min, max : integer);                {func  7}
Procedure mRowRange(min, max : integer);                {func  8}
Procedure mGraphCursor(hHot, vHot : integer;
                       maskSeg, maskOfs : word);        {func  9}
Procedure mTextCursor(ctype, p1, p2 : word);            {func 10}
Procedure mMotion(var moved : moveRec);                 {func 11}
Procedure mInstTask(mask, taskSeg, taskOfs : word);     {func 12}
Procedure mLpenOn;                                      {func 13}
Procedure mLpenOff;                                     {func 14}
Procedure mRatio(horiz, vert : integer);                {func 15}
{---------------------------------------------------------------}

Implementation

{................... lower .....................................}
Function lower(n1, n2 : integer) : integer;  { local to unit}
begin
  if n1 < n2 then lower := n1
  else lower := n2;
end;
{................... upper ......................................}
Function upper(n1, n2 : integer) : integer;  { local to unit}
begin
  if n1 > n2 then upper := n1
  else upper := n2;
end;
{================== mReset =======================================}
Procedure mReset;   { reset mouse to default condition }
begin
  reg.ax := 0;
  intr(MDD, reg);
  if reg.ax <> 0 then
    mouse.exists := true
  else
    mouse.exists := false;
  if mouse.exists then mouse.nButtons := reg.bx;
end;
{=================== mShow =======================================}
Procedure mShow;    { make mouse cursor visible }
begin
  reg.ax := 1;
  intr(MDD, reg);
  reg.ax := 1;
  intr(MDD, reg);
end;
{==================== mHide ======================================}
Procedure mHide;    { make mouse cursor invisible }
begin
  reg.ax := 2;
  intr(MDD, reg);
end;
{==================== mPos =======================================}
Procedure mPos;     { get mouse status and position }
begin
  reg.ax := 3;
  intr(MDD, reg);
  mouse.buttonStatus := reg.bx;
  mouse.column := reg.cx;
  mouse.row := reg.dx;
end;
{====================== mTxtPos ==================================}
Procedure mTxtPos;  { adjusts mPos for 80 x 25 text work }
begin
  mPos(mouse);
  mouse.row := (mouse.row + 8) DIV 8;
  mouse.column := (mouse.column + 8) DIV 8;
end;
{==================== mMoveto ====================================}
Procedure mMoveto;   { move mouse cursor to new location }
begin
  reg.ax := 4;
  reg.cx := col;
  reg.cx := row;
  intr(MDD, reg);
end;
{======================= mTxtMoveto =============================}
Procedure mTxtMoveto; { move mouse cursor to text location }
begin
  col := (col * 8) - 8;
  row := (row * 8) - 8;
  mMoveto(col,row);
end;
{==================== mPressed ===================================}
Procedure mPressed;  { get pressed info about a particular button }
begin
  reg.ax := 5;
  reg.bx := button;
  intr(MDD, reg);
  mouse.buttonStatus := reg.ax;
  mouse.opCount := reg.bx;
  mouse.column := reg.cx;
  mouse.row := reg.dx;
end;
{===================== mPressedT ================================}
Procedure mPressedT; { get pressed info ... in text environment}
begin
  mPressed(button,mouse);
  mouse.row := (mouse.row + 8) DIV 8;
  mouse.column := (mouse.column + 8) DIV 8;
end;
{==================== mReleased =================================}
Procedure mReleased;  { get released info about a button }
begin
  reg.ax := 6;
  reg.bx := button;
  intr(MDD, reg);
  mouse.buttonStatus := reg.ax;
  mouse.opCount := reg.bx;
  mouse.column := reg.cx;
  mouse.row := reg.dx;
end;
{======================== mReleasedT =============================}
Procedure mReleasedT; { get released info about a button in text environ}
begin
  mReleased(button,mouse);
  mouse.row := (mouse.row + 8) DIV 8;
  mouse.column := (mouse.column + 8) DIV 8;
end;
{===================== mColRange =================================}
Procedure mColRange;  { set column range for mouse }
begin
  reg.ax := 7;
  reg.cx := lower(min, max);
  reg.dx := upper(min, max);
  intr(MDD, reg);
end;
{====================== mRowRange =================================}
Procedure mRowRange;   { set row range for mouse }
begin
  reg.ax := 8;
  reg.cx := lower(min, max);
  reg.dx := upper(min, max);
  intr(MDD, reg);
end;
{======================== mGraphCursor =============================}
Procedure mGraphCursor;     { set mouse graphics cursor }
begin
  reg.ax := 9;
  reg.bx := hHot;
  reg.cx := vHot;
  reg.dx := maskOfs;
  reg.es := maskSeg;
  intr(MDD, reg);
end;
{======================= mTextCursor ================================}
Procedure mTextCursor;      { set mouse text cursor  }
  {----------------------------------------------------}
  {   Notes:                                           }
  { Type 0 is the software cursor. When specified, p1  }
  {   and p2 are the screen and cursor masks.          }
  { Type 1 is the hardware cursor.  When specified, p1 }
  {   and p2 are the start and stop scan lines, i.e.   }
  {   the cursor size and shape.                       }
  {----------------------------------------------------}
begin
  reg.ax := 10;
  reg.bx := ctype;
  reg.cx := p1;
  reg.dx := p2;
  intr(MDD, reg);
end;
{===================== mMotion ====================================}
Procedure mMotion;      { Net movement of mouse since last call    }
                        { unit of measures = "mickeys" (1/100 inch)}
begin
  reg.ax := 11;
  intr(MDD, reg);
  moved.hCount := reg.cx;
  moved.vCount := reg.dx;
end;
{======================= mInstTask ================================}
Procedure mInstTask;    { Install user-defined task }
   {---------------------------------------------------}
   { Notes:                                            }
   { mask is 16 bit value, but you only use bits 0 - 6 }
   {  bit         event                                }
   {   0          mouse cursor moved                   }
   {   1          left button pressed                  }
   {   2          left button released                 }
   {   3          right button pressed                 }
   {   4          right button released                }
   {   5          middle button pressed (logitech only)}
   {   6          middle button release (logitech only)}
   {                                                   }
   { when driver detects activity and calls handler,   }
   { the following info is summarized and passed:      }
   {  ax    event that occured (same bits as above)    }
   {  bx    status of buttons ( 1 bit if down 0 for up)}
   {  cx    horizontal cursor position                 }
   {  dx    vertical cursor position                   }
   {---------------------------------------------------}
begin
  reg.ax := 12;
  reg.cx := mask;
  reg.dx := taskOfs;
  reg.es := taskSeg;
  intr(MDD, reg);
end;

{======================== mLpenOn ================================}
Procedure mLpenOn;    { turn on light pen emulation (default) }
begin
  reg.ax := 13;
  intr(MDD, reg);
end;
{======================== mLpenOff ================================}
Procedure mLpenOff;    { turn off light pen emulation }
begin
  reg.ax := 14;
  intr(MDD, reg);
end;
{======================== mRatio ===================================}
Procedure mRatio;      { set mickey to pixel ratio }
  {-------------------------------------------------}
  { Notes:                                          }
  { Ratios are R/8.                                 }
  { Default is 16 for vertical, 8 for horizontal    }
  {-------------------------------------------------}
begin
   reg.ax := 15;
   reg.cx := horiz;
   reg.dx := vert;
end;
{--------------------------------------------------------------------}
end.
